/**
 * 月度表单逻辑层 
 * 作者: 陈金晶
 * 月度表单相关逻辑 包括孵化器月度填报的增删改查
 */
import { ERRORENUM } from "../../config/errorEnum";
import { BizError } from "../../util/bizError";
import * as monthData from "../../data/fuHuaQi/monthTable";
import { extractData, getTaskId } from "../../util/tools";
import moment = require("moment");
import { findAllFuHuaQiOperationNameMap } from "../../data/fuHuaQi/fuhuaqi";
import { MonthListConfig } from "../../config/ojbectResultKeyConfig";
import { TASKTYPEENUM } from "../../config/enum";

/**
 * 新添加孵化器月度填报
 * @param uscc 孵化器的统一信用代码
 * @param occupancyRate 本月出租率
 * @returns {isSuccess:true/false}
 */
export async function createReport(uscc:string, occupancyRate:number) {
    const TaskId = getTaskId(uscc);
    let dataBaseInfo = await monthData.findmonthTableByTaskId(TaskId);

    if (dataBaseInfo) throw new BizError(ERRORENUM.该孵化器月度填报已存在, `${uscc}重复提交了月度填报值为${occupancyRate}`);

    const MonthNumber = moment().subtract(1, 'months').month() + 1;
    const MonthTableName = `${MonthNumber}月孵化器月度填报`;
    await monthData.addOnceReport(TaskId, MonthTableName, uscc, occupancyRate);
    
    return {isSuccess:true};
}

/**
 * 修改孵化器月度报表
 * @param uscc 孵化器的统一信用代码
 * @param occupancyRate 本月出租率
 * @returns {isSuccess:true/false}
 */
export async function updateReport(uscc:string, occupancyRate:number) {
    const TaskId = getTaskId(uscc);
    let dataBaseInfo = await monthData.findmonthTableByTaskId(TaskId);
    if (!dataBaseInfo || !dataBaseInfo.taskId) throw new BizError(ERRORENUM.未找到数据, `未找到${uscc}的月度报表` );
    
    dataBaseInfo.occupancyRate = occupancyRate;
    await dataBaseInfo.save();

    return {isSuccess:true};
}

/**
 * 删除孵化器月度报表
 * @param uscc 孵化器的统一信用代码
 * @returns  {isSuccess:true/false}
 */
export async function deleteReport(uscc:string) {
    const TaskId = getTaskId(uscc);
    let dataBaseInfo = await monthData.findmonthTableByTaskId(TaskId);
    if (!dataBaseInfo || !dataBaseInfo.taskId) throw new BizError(ERRORENUM.未找到数据);

    await monthData.deleteReport(uscc, dataBaseInfo.name);

    return {isSuccess:true};
}


/**
 * 月度出租率数据列表
 * 管理后台使用
 * @param state 填报状态
 * @param year 数据年份
 * @param month 数据月份
 * @param page 页面
 */
export async function monthTableList(state:number, year:number, month:number, page:number) {
    let selectParam:any = {};
    if (year) selectParam.year = year;
    if (month) selectParam.month = month;

    // findTaskListByType(TASKTYPEENUM.孵化器月度填报);

    let dataBaseList = await monthData.findMonthTableList(selectParam, (page-1)*10 );
    let count = await monthData.findMonthTableListCount(selectParam);

    let operationNameMap = findAllFuHuaQiOperationNameMap();

    let dataList = [];
    dataBaseList.forEach( info => {
        let onceInfo = {
            operationName:operationNameMap[info.fuHuaQiUscc],
            state:'已填报',
            time:`${info.year}年${info.month}月`,
            occupancyRate:info.occupancyRate
        };
        dataList.push(onceInfo);
    });


    //==


    return {count, dataList}
}



/**
 * 根据uscc查询月度报表 
 * @param uscc 企业统一信用代码
 */
 export async function getMonthByUscc(uscc:string) {
    const TaskId = getTaskId(uscc);
    let dataBaseInfo = await monthData.findmonthTableByTaskId(TaskId);
    if (!dataBaseInfo || !dataBaseInfo.taskId) throw new BizError(ERRORENUM.未找到数据, `未找到${uscc}的月度报表` );
    let data = extractData(MonthListConfig, dataBaseInfo);

    return {data};
}